	Subroutine Projection

	use Def_Par
	use Def_Proj
	use Def_moments
	use MNR
	use QNP
	use gh_def
	use Errors

	use DFPORT
	use DFLIB
	use IMSLF90 

	Integer(4) i, retcode, trials, iter, GetNC
	Real(8) temp, crit(1:6), Func1, Func2, Func3, Func4
	Real(8), allocatable :: gamma(:), gamma2(:)

	External Func1, Func2, Func3, Func4
	External Sys_Proj1a, Sys_Proj1b, Sys_Proj2a, Sys_Proj2b, Sys_Proj3a, Sys_Proj3b

C Read additional information for the alogrithm from Parameters.txt
	Call GetI('psi_type','Parameters.txt',psi_type)		
	Call GetI('proj_type','Parameters.txt',proj_type)	
	Call GetI('trials','Parameters.txt',trials)
	Call GetL('psi_initial','Parameters.txt',psi_initial)		
	Call GetI('k_nodes','Parameters.txt',k_nodes)
	Call GetI('z_nodes','Parameters.txt',z_nodes)
	

	Select Case (psi_type)
		Case(1)  ! complete polynomial
			Call GetI('psi_degree','Parameters.txt',psi_degree)
			ngamma=GetNC(psi_degree-1)
		Case(2)  ! product base polynomial
			Call GetI('psi_degree1','Parameters.txt',psi_degree1)
			Call GetI('psi_degree2','Parameters.txt',psi_degree2)
			ngamma=psi_degree1*psi_degree2
	End Select

	Call GetD('psi_kmin','Parameters.txt',psi_kmin)  ! bounds used for psi
	Call GetD('psi_kmax','Parameters.txt',psi_kmax)

	psi_bounds(1,1)=dlog(psi_kmin*kstar) ! note: bounds for psi in logs of k and z
	psi_bounds(1,2)=dlog(psi_kmax*kstar)
	
	Call GetD('k_min','Parameters.txt',k_min)   ! bounds on the capital stock
	Call GetD('k_max','Parameters.txt',k_max)
	x_bounds(1,1)=k_min*kstar  ! bounds for k and z for integration not in logs
	x_bounds(1,2)=k_max*kstar
	
	Call GetD('z_min','Parameters.txt',z_min)  ! bounds used for z
	Call GetD('z_max','Parameters.txt',z_max)	
	
	x_bounds(2,1)=z_min
	x_bounds(2,2)=z_max

	temp=2.0d0

	psi_bounds(2,1)=1.05*(rho*dlog(z_min)+dsqrt(temp)*sigma*ghx(1))
	psi_bounds(2,2)=1.05*(rho*dlog(z_max)+dsqrt(temp)*sigma*ghx(4))



C allocate space for Sys_Pe
	ALLOCATE(gamma(1:ngamma), gamma2(1:ngamma),gamma0(1:ngamma))
	

C Initialize parameters
	Lsec(1)=RTC()
	iter=1
1	Call GetInitialGamma(gamma)
	
	Call GetI('mnr_maxit','Parameters.txt',mnr_maxit)

	crit(1)=1.0d0	
	TypF=Func1(ngamma,gamma)
	QN_GradTol=1.0d-6

	Select Case(Proj_type)

		Case(1) ! least squares projection
			Call QuasiNewton(Func1,ngamma,gamma,gamma2,crit(1:4))
			write(lfh,"('Termination code from QuasiNewton= ',F3.0)") crit(1)
			write(lfh,"('Maximum abs. value of gradient   = ',F20.10)") crit(2)
			write(lfh,"('f(x*)                            = ',F20.10)") crit(3)
			write(lfh,"('Number of iterations             = ',F10.0)")  crit(4)	
			
		Case(2) ! Galerkin projection		
			Select Case(algo)
				Case(1)					
					Select Case(psi_type)
						Case(1) ! complete polynomial
							Call FixvMN1(ngamma,gamma,Sys_Proj3a,gamma2,crit)
						Case(2) ! product base polynomial
							Call FixvMN1(ngamma,gamma,Sys_Proj2a,gamma2,crit)
					End Select
					write(lfh,"('Exit from FixvMN1 with message: ',A50,' FError= ',I2)") MNR_Message(DINT(crit(1))), FError				
					write(lfh,"('max|f(x)| at solution:          ',F20.10)") crit(2)
					write(lfh,"('number of iterations:           ',F5.0)") crit(6)	
					if (FError.ne.0) crit(1)=1.0
				Case(2)	
					i=(ngamma*(3*ngamma+13))/2.
					crit(6)=1.e-8
					allocate(yt(1:i))
					gamma2=gamma
					Select Case(psi_type)
						Case(1) ! complte polynomial	
							Call hybrd1(Sys_Proj3b,ngamma,gamma2,gamma,crit(6),retcode,yt,i)
						Case(2) ! product base polynomial
					        Call hybrd1(Sys_Proj2b,ngamma,gamma2,gamma,crit(6),retcode,yt,i)							
					End Select
					write(lfh,"('Exit code from hybrd1: ',I3,' FError= ',I3)") retcode, FError
					if (retcode.eq.1) crit(1)=0
					deallocate(yt)
			End Select
	
		Case(3) ! collocation
			Select Case(algo)
				Case(1)					
					Call FixvMN1(ngamma,gamma,Sys_Proj1a,gamma2,crit)
					write(lfh,"('Exit from FixvMN1 with message: ',A50,' FError= ',I2)") MNR_Message(DINT(crit(1))), FError
					write(lfh,"('Exit from FixvMN1 with message: ',A50,' FError= ',I2)") MNR_Message(DINT(crit(1))), FError				
					write(lfh,"('max|f(x)| at solution:          ',F20.10)") crit(2)
					write(lfh,"('number of iterations:           ',F5.0)") crit(6)	
					if (FError.ne.0) crit(1)=1.0
				Case(2)	
					i=(ngamma*(3*ngamma+13))/2.
					crit(6)=1.e-8
					allocate(yt(1:i))
					gamma2=gamma
					Call hybrd1(Sys_Proj1b,ngamma,gamma2,gamma,crit(6),retcode,yt,i)
					write(lfh,"('Exit code from hybrd1: ',I3,' FError= ',I3)") retcode, FError
					if (retcode.eq.1) crit(1)=0
					deallocate(yt)
			End Select
	
	End Select
	if ((crit(1).ne.0) .and. (psi_initial.eq.2)) then
		iter=iter+1		
		if (iter.le.trials) goto 1	
	endif

	Lsec(1)=RTC()-Lsec(1)	                 

	if (crit(1).ne.0) then
		ErrorCode=1
	else
C Save solution vector
		open(25,file='Proj_Par.bin',form='unformatted')
		write(25) gamma2
		close(25)
		gamma0=gamma2	
	endif

	Return
	End Subroutine



C *********************** Subroutines ***********************************
C
C lambda=Psi(Par,K,Z) returns the estimate of
C the right-hand side of the Euler equation, given
C observations on Z and K and the current values of
C the parameters in gamma
C
      Real(8) Function Psi(gamma,K,Z)

	use Def_Par
	use Def_Proj
	use Errors
		
	REAL(8) K, Z, gamma(1:ngamma)
	
	
	Integer(4) l, degree(1:2)
	
	Real(8) x(1:2), ChebEval2, ChebEval3

	x(1)=dlog(k)
	x(2)=dlog(z)
	if ( (x(1).lt.psi_bounds(1,1)).or.(x(1).gt.psi_bounds(1,2))) then
		FError=1
		Return
	Endif
	if ((x(2).lt.psi_bounds(2,1)) .or. (x(2).gt.psi_bounds(2,2))) then
		FError=2
		write(lfh,"('z= ',F15.5,' dlog(z)= ',F15.5,' zmin=',F15.10,' zmax=',F15.10)") z, x(2), psi_bounds(2,1), psi_bounds(2,2)
		Return
	Endif	
	
	Select Case(psi_type)	
		Case(1) ! complete polynomial
			l=psi_degree-1			
			Psi=ChebEval2(gamma,l,x,psi_bounds)

		Case(2) ! product base polynomial
			degree(1)=psi_degree1
			degree(2)=psi_degree2
			Psi=ChebEval3(gamma,degree,x,psi_bounds)

	End Select
	
	if ((psi.gt. 700.0d0) .or. (psi.lt.-700) .or. (isNan(psi))) then
		FError=3
		return
	endif
	psi=beta*dexp(psi)
			
	Return
	END FUNCTION
C**********************************************************************************************
C
C GetNC(p): Compute the number of coefficients of a complete polynomial in two
C           independent variables of degree p, where p=0 is a constant

	Integer(4) Function GetNC(p)

	Integer(4) p

	Integer(4) i1, i2
	GetNC=0	
	Do i1=0,p
		Do i2=0,(p-i1)
			GetNC=GetNC+1	
		End Do
	End Do

	Return

	End Function

C**********************************************************************************************************
C
C GetInitialGamma: Initializes the parameter vector for the non-linear equations solver
C
C	
	Subroutine GetInitialGamma(gamma)

	use Def_Proj
	use Def_Par
	use Errors
	use GSPars1

	Real(8) gamma(1:ngamma)

	Integer(4) n, i, nodes(1:2), degree(1:2)
	Real(8) Fit, Func1, Func2, Fun3, Func4, Func5
	Real(8), allocatable :: gammai(:)

	Character(40) infile

	Logical ok

	External Func1, Func2, Func3, Func4, Func5

	Select Case (psi_initial)

		Case(1) ! Read from file
			Call GetI('psi_increase','Parameters.txt',psi_increase)
			Call GetS('psi_initial_file','Parameters.txt',infile)
			inquire(file=infile,exist=ok)			
			if (.not. ok) then
				Call MyMessage("File does not exist. Program terminates","PE Erorr")
				stop
			endif
			open(25,form='unformatted',file=infile)
			Select Case(psi_increase)
				Case(1)
					read(25) gamma
				Case(2)
					n=((psi_degree-1)*psi_degree)/2
					allocate(gammai(1:n))
					read(25) gammai
					Call Increase2(n,ngamma,gammai,gamma)
				Case(3)
					n=(psi_degree1-1)*psi_degree2
					allocate(gammai(1:n))
					read(25) gammai
					i=1
					Call Increase1(i,n,ngamma,gammai,gamma)
				Case(4)			
					n=psi_degree1*(psi_degree2-1)
					allocate(gammai(1:n))
					read(25) gammai
					i=2
					Call Increase1(i,n,ngamma,gammai,gamma)				
			End Select			
			close(25)				

		Case(2) ! Use genetic search to find initial parameter vector			  
			Call GetI('npop','Parameters.txt',npop)
			Call GetI('ngen','Parameters.txt',ngen)
			Call GetD('s_sigma','Parameters.txt',s_sigma)
			Select Case(proj_type)
				Case(1)
					Call GSearch1(Func1,ngamma,gamma,Fit) 
				Case(2)
					Select Case(psi_type)
						Case(1) ! complete polynomial
							Call GSearch1(Func4,ngamma,gamma,Fit)
						Case(2) ! product base polynomial
							Call GSearch1(Func3,ngamma,gamma,Fit)
					End Select					
				Case(3)
					Call GSearch1(Func2,ngamma,gamma,Fit)
			End Select
			Write(lfh,"('Initial parameters form genetic search:')")
			do i=1,ngamma,1
				Write(lfh,"(F15.10))") gamma(i)
			end do
			Write(lfh,"('Function value of solution ',F15.10)") Fit

		Case(3) ! use the linear solution coefficients and ChebCoef2
			Call GetD('llx','Parameters.txt',llx)
			Call GetD('llz','Parameters.txt',llz)
			Select Case(psi_type)
				Case(1) ! complete polynomial
					nodes(1)=k_nodes
					degree(1)=psi_degree-1
					Call ChebCoef2b(Func5,degree(1),nodes(1),x_bounds,ngamma,Gamma)	                                
				Case(2) ! product base polynomial
					nodes(1)=k_nodes
					nodes(2)=z_nodes
					degree(1)=psi_degree1
					degree(2)=psi_degree2
					Call ChebCoef2a(Func5,degree,nodes,x_bounds,Gamma)  
			End Select
						
	End Select 

	Return

	End Subroutine
C*******************************************************************************************************
C
C GetKCM(k,z,k1,c,mu)
C
C given k and z, compute lambda=psi(gamma,k,Z), then c, and k1 and return
C solution.
                 
	Subroutine GetKCM(k,z,k1,c,mu)

	use Def_Par
	use Def_Proj
	use Errors

	Real(8) k, z, k1, c, mu

	Real(8) rhs, Psi

	rhs=Psi(gamma0,k,z)	
	if (FError.ne.0) return
	c=rhs**(-1.0d0/eta)	
	k1=z*(k**alpha)+(1.0d0-delta)*k-c
	
c	Select Case(proj_type)
c		Case(3)
c			if ((k1.lt.x_bounds(1,1)) .or. (k1.gt.x_bounds(1,2))) then
c				FError=4
c				write(lfh,"('k= ', F10.5,' kmin= ',F10.5,' kmax= ',F10.5)") k1, x_bounds(1,1), x_bounds(1,2)
c				return
c			endif
c	End Select	

	if (k1.lt.(1.0d0-delta)*k) then
		c=z*(k**alpha)
		k1=(1.d0-delta)*k
		mu=c**(-eta)-rhs
	else
		mu=0.0d0
	endif

	Return

	End Subroutine

	
C************************************************************************************************
C
C Resid(k,z) returns the residual at the point (z,k), the current vector of
C        parameters gvec must be given.
C        The parameter vector gamma is defined in def_proj

      Real(8) Function Resid(k,z)

      use Def_Par
	use Def_Proj
	use Errors

	Real(8) k, z

      Real(8) l, c, rhs
	Real(8) Psi, GH_INT1, GetRhs
	
	External GetRHS, GH_INT1, Psi


	l=Psi(gamma0,k,z)	
	if (FError.ne.0) return	
	c=l**(-1.0d0/eta)	
	k_ex=z*(k**alpha)+(1.0d0-delta)*k-c
	
c	Select Case(proj_type)
c		Case(3)
c			if ((k_ex.lt.x_bounds(1,1)) .or. (k_ex.gt.x_bounds(1,2))) then
c				FError=4
c				return
c			endif
c	End Select
   
	z_ex=rho*dlog(z)

	FError=0
	rhs=GH_INT1(GetRhs,sigma)				
	if (FError.ne.0) return
	rhs=rhs*beta

	Resid=rhs-l

	Return

	End Function

C**********************************************************************************************
C
C ResidSQ(k,z) returns Resid(k,z)*Resid(k,z)
	Real(8) Function ResidSQ(k,z)

	use Errors

	Real(8) k, z

	Real(8) temp, Resid

	FError=0
	temp=Resid(k,z)
	If (FError.ne.0) return

	ResidSQ=temp*temp

	Return

	End Function

C********************************************************************************************
C
C fx=Func1(n,x): The minimizer of this function is the least squares solution

	Real(8) Function Func1(n,x)

	use Def_Proj
	use Errors

	Integer(4) n
	Real(8) x(1:n)

	Integer(4) nodes(1:2)

	Real(8) ResidSQ, GC_INT2

	External ResidSQ, GC_INT2

	nodes(1)=k_nodes
	nodes(2)=z_nodes

	gamma0=x

	Func1=GC_Int2(ResidSQ, x_bounds, nodes)
	if (FError.ne.0) Func1=-1

	Return

	End Function

C********************************************************************************************************
C
C Sys_Proj1a(n,m,x,fx) the rhs of the system of equations whose zero is the collocation solution
C                      requires a product base polynomial

	Subroutine Sys_Proj1a(m,n,x,fx)

      use Def_Proj	
	use Errors
	use IMSLF90	

	Integer(4) m, n
	Real(8) x(1:n), fx(1:m)


      Integer(4) i, j, k
	Real(8) z1(psi_degree1),z2(1:psi_degree2), pi, Dconst, Resid

	External Resid

	pi=Dconst('pi')    
        
	
      Do i=1,psi_degree1
	   z1(i)=dcos(((dble(2.0d0*i)-1.0d0)/(dble(2.0d0*psi_degree1)))*pi)
	   z1(i)=0.5d0*(z1(i)+1.0d0)*(x_bounds(1,2)-x_bounds(1,1)) + x_bounds(1,1)
	End Do
	
	Do i=1,psi_degree2
	   z2(i)=dcos(((dble(2.0d0*i)-1.0d0)/(dble(2.0d0*psi_degree2)))*pi)   
	   z2(i)=0.5d0*(z2(i)+1.0d0)*(x_bounds(2,2)-x_bounds(2,1)) + x_bounds(2,1)
	End Do

	
	gamma0=x
      
	k=1
	FError=0
      Do i=1,psi_degree1,1
            Do j=1,psi_degree2,1
              fx(k)=Resid(z1(i),z2(j))
			if (FError.ne.0) return
              k=k+1      
            End Do
      End Do
      

      Return

	End Subroutine

C********************************************************************************************************
C
C Sys_Proj1b: the same as Sys_Proj1a but with different calling, used for Hybrid1
C
	Subroutine Sys_Proj1b(n,x,fx,retcode)
	
	use Errors
	
	Integer(4) n, retcode
	Real(8) x(1:n), fx(1:n)
	
	Call Sys_Proj1a(n,n,x,fx)
	
	If (FError.ne.0) retcode=Ferror
	
	return

	End Subroutine


C********************************************************************************************************
C
C	Func2(n,x): returns the maximum absolute value of Sys_Proj1a,
C                 used to find starting values for the collocation solution
C
	Real(8) Function Func2(n,x)

	use Errors

	Integer(4) n
	Real(8)    x(1:n)
	
	Real(8)    fx(1:n)

	FError=0
	Call Sys_Proj1a(n,n,x,fx)
	If (FError.ne.0) then
		Func2=-1.0
	else
		Func2=MaxVal(dabs(fx))
	endif

	Return

	End Function

C*************************************************************************************************
C 
C Sys_Proj2a(m,n,x,fx)
C
C defines the rhs of the system of non-linear equation whose zero is
C the Galerkin projection solution of the model.

      Subroutine Sys_Proj2a(m,n,x,fx)

	use Def_Par
	use Def_Proj
	use IMSLF90	
      use Errors

	Integer(4) m, n
	Real(8)  x(1:n), fx(1:m)
      
      Integer(4) i, i1, i2,  k, j1, j2
	Real(8) z1(1:k_nodes,1:2), z2(1:z_nodes,1:2)
	Real(8) pi, Dconst, Resid, Cheb, r	

	pi=Dconst('pi')    

C Computation of integration nodes        
      Do i=1,k_nodes
	   z1(i,1)=dcos(((dble(2.0d0*i)-1.0d0)/(dble(2.0d0*k_nodes)))*pi) ! zeros of first base function
	   z1(i,2)=0.5d0*(z1(i,1)+1.0d0)*(x_bounds(1,2)-x_bounds(1,1)) + x_bounds(1,1)
	End Do
	
	Do i=1,z_nodes
	   z2(i,1)=dcos(((dble(2.0d0*i)-1.0d0)/(dble(2.0d0*z_nodes)))*pi) ! zeros of second base  function
         z2(i,2)=0.5d0*(z2(i,1)+1.0d0)*(x_bounds(2,2)-x_bounds(2,1))+x_bounds(2,1)
	End Do
      
	gamma0=x
      
	fx=0.0
	
      Do i1=1,k_nodes
           Do i2=1,z_nodes
                  r=Resid(z1(i1,2),z2(i2,2))
	            if (FError.ne.0) write(lfh,"(2(F10.4,2X),1X,I)") z1(i1,2), z2(i2,2), FError
	            If (FError.ne.0) Return
	            k=1
	            Do j1=1,psi_degree1
	               Do j2=1,psi_degree2	                  
				         fx(k)=fx(k)+r*Cheb(j1-1,z1(i1,1))*Cheb(j2-1,z2(i2,1))*
     +                     dsqrt(1.0d0-(z1(i1,1)**2))*dsqrt(1.0d0-(z2(i2,1)**2))
	                     k=k+1
	               End Do
                  End Do	            
	     End Do
	End Do
	fx=fx*(0.5d0*pi)*(0.5d0*pi)*(x_bounds(1,2)-x_bounds(1,1))*
     +   (x_bounds(2,2)-x_bounds(2,1))*(1.0d0/dble(2.0d0*z_nodes))*(1.0d0/dble(2.0d0*k_nodes))
	Return

	End Subroutine

C********************************************************************************************************
C
C Sys_Proj2b: the same as Sys_Proj2a but with different calling, used for Hybrid1
C
	Subroutine Sys_Proj2b(n,x,fx,retcode)
	
	use Errors
	
	Integer(4) n, retcode
	Real(8) x(1:n), fx(1:n)
	
	Call Sys_Proj2a(n,n,x,fx)
	
	If (FError.ne.0) retcode=Ferror
	
	return

	End Subroutine


C********************************************************************************************************
C
C	Func3(n,x): returns the maximum absolute value of Sys_Proj1a,
C                 used to find starting values for the collocation solution
C
	Real(8) Function Func3(n,x)

	use Errors

	Integer(4) n
	Real(8)    x(1:n)
	
	Real(8)    fx(1:n)

	FError=0
	Call Sys_Proj2a(n,n,x,fx)
	If (FError.ne.0) then
		Func3=-1.0
	else
		Func3=MaxVal(dabs(fx))
	endif

	Return

	End Function

C*************************************************************************************************
C 
C Sys_Proj3a(m,n,x,fx)
C
C defines the rhs of the system of non-linear equation whose zero is
C the Galerkin projection solution of the model if a complete polynomial was chosen

      Subroutine Sys_Proj3a(m,n,x,fx)

	use Def_Par
	use Def_Proj
	use IMSLF90	
      use Errors

	Integer(4) m, n
	Real(8)  x(1:n), fx(1:m)
      
      Integer(4) i, i1, i2,  k, j1, j2
	Real(8) z1(1:k_nodes,1:2), z2(1:z_nodes,1:2)
	Real(8) pi, Dconst, Resid, Cheb, r	

	pi=Dconst('pi')    

C Computation of integration nodes        
      Do i=1,k_nodes
	   z1(i,1)=dcos(((dble(2.0d0*i)-1.0d0)/(dble(2.0d0*k_nodes)))*pi) ! zeros of first base function
	   z1(i,2)=0.5d0*(z1(i,1)+1.0d0)*(x_bounds(1,2)-x_bounds(1,1)) + x_bounds(1,1)
	End Do
	
	Do i=1,z_nodes
	   z2(i,1)=dcos(((dble(2.0d0*i)-1.0d0)/(dble(2.0d0*z_nodes)))*pi) ! zeros of second base  function
         z2(i,2)=0.5d0*(z2(i,1)+1.0d0)*(x_bounds(2,2)-x_bounds(2,1))+x_bounds(2,1)
	End Do
      
	gamma0=x
      
	fx=0.0
	
      Do i1=1,k_nodes
           Do i2=1,z_nodes
                  r=Resid(z1(i1,2),z2(i2,2))
	            if (FError.ne.0) write(lfh,"(2(F10.4,2X),1X,I)") z1(i1,2), z2(i2,2), FError
	            If (FError.ne.0) Return
	            k=1
	            Do j1=1,psi_degree,1
	               Do j2=1,(psi_degree+1-j1),1	                  
				         fx(k)=fx(k)+r*Cheb(j1-1,z1(i1,1))*Cheb(j2-1,z2(i2,1))*
     +                     dsqrt(1.0d0-(z1(i1,1)**2))*dsqrt(1.0d0-(z2(i2,1)**2))
	                     k=k+1
	               End Do
                  End Do	            
	     End Do
	End Do
	fx=fx*(0.5d0*pi)*(0.5d0*pi)*(x_bounds(1,2)-x_bounds(1,1))*
     +   (x_bounds(2,2)-x_bounds(2,1))*(1.0d0/dble(2.0d0*z_nodes))*(1.0d0/dble(2.0d0*k_nodes))
	Return

	End Subroutine

C********************************************************************************************************
C
C Sys_Proj3b: the same as Sys_Proj2a but with different calling, used for Hybrid1
C
	Subroutine Sys_Proj3b(n,x,fx,retcode)
	
	use Errors
	
	Integer(4) n, retcode
	Real(8) x(1:n), fx(1:n)
	
	Call Sys_Proj3a(n,n,x,fx)
	
	If (FError.ne.0) retcode=Ferror
	
	return

	End Subroutine


C********************************************************************************************************
C
C	Func4(n,x): returns the maximum absolute value of Sys_Proj1a,
C                 used to find starting values for the collocation solution
C
	Real(8) Function Func4(n,x)

	use Errors

	Integer(4) n
	Real(8)    x(1:n)
	
	Real(8)    fx(1:n)

	FError=0
	Call Sys_Proj3a(n,n,x,fx)
	If (FError.ne.0) then
		Func4=-1.0
	else
		Func4=MaxVal(dabs(fx))
	endif

	Return

	End Function


C****************************************************************************************************
C
C  Increase1(what,no,nn,old,new)
C
C Given the old parameter vector old, increase creates a new
C parameter vector by filling the missing coefficients with zeros.
C What=1, 2 determines whether the first or the second base function
C coefficients are filled with zeros. no is the old nn is the new
C number of coefficients.	

      Subroutine Increase1(what,no,nn,old,new)

      use Def_Proj

	Integer(4) what, no, nn
      Real(8) old(no), new(nn)

      Integer(4) ko, kn, i, j

	ko=1
	kn=1

	Select case(what)

	   Case(1)	     
	     Do i=1,psi_degree1-1
	        Do j=1,psi_degree2
	           new(kn)=old(ko)
	                kn=kn+1
	                ko=ko+1
	        End Do
	     End Do
	     Do j=1,psi_degree2
	        new(kn)=0.0
	            kn=kn+1
	     End Do

          Case(2)
	      Do i=1,psi_degree1
		     Do j=1,psi_degree2-1
			    new(kn)=old(ko)
				     ko=ko+1
	                 kn=kn+1
			 end do
			    new(kn)=0.0
				    kn=kn+1
	      End do
		  
      End Select
	
	Return
	
	End Subroutine				 

C****************************************************************************************************
C
C  Increase2(no,nn,old,new)
C
C Given the old parameter vector old, increase creates a new
C parameter vector by filling the missing coefficients with zeros.
C no is the old nn is the new  number of coefficients.	

      Subroutine Increase2(no,nn,old,new)

      use Def_Proj

	Integer(4) no, nn
      Real(8) old(1:no), new(1:nn)

      Integer(4) k, i, j, p1

	k=1	
	new=0.0d0
	p1=psi_degree-1
	Do i=1,p1,1
		Do j=1,(p1+1-i)
			new(k)=old(k)
	        k=k+1	                
	     End Do
	End Do
	
	Return
	
	End Subroutine				 


C*********************************************************************************************
C
C Func5(k,z) is used to initialize the polynomial from the linear solution
C
      Real(8) Function Func5(K,Z)

	use Def_Par

	Real(8) K, Z	
      

	Func5=lstar+ Llx*(k-kstar)+Llz*(z-1.d0)

	Return

	End Function
