C SGNNI_c: Use the parameterized expectations approach to solve
C          the stochastic growth model with non-negative investment
C          

C Definition of module that provides parameters of the model
C and parameters of the solution techique

      Module Def_Par

C Parameters of the model
	Real(8) alpha, beta, delta, eta, rho, sigma

C Stationary solution
	Real(8) yk, kstar, cstar, istar, ystar, lstar, zstar
	Real(8) llx, llz

C Solution technique
	Integer(4) algo

C Parameter to determine run time
      Real(8) Lsec(1:3)

	Character(256) Laufzeit
	Character(18) Laufzeiten(1:3)

	DATA Lsec/3*0.0/
	DATA Laufzeiten(1)/'Solution:        '/	
	DATA Laufzeiten(2)/'Euler residuals: '/
	DATA Laufzeiten(3)/'DM statistic:    '/

	
	End Module Def_Par

C Definitions used for writing results
	Module Def_Output

	Character(30) OutFile		
	Character(50) MyDate	

	External MyDate

	End Module Def_Output

C Definition of module that provides the information for the computation of second moments
	Module Def_Moments

	Integer(4) nobs, nofs

	Real(8), allocatable :: kt(:), zt(:), ct(:), yt(:), mut(:)		
	Real(8), allocatable :: eps_sim(:,:)
	Logical compute_moments, new_shocks

	End Module Def_Moments

C Defintion of module that provides the information for the computation of the Euler equations residuals
	Module Def_Euler

	Logical compute_euler
	Integer(4) eu_nz, eu_nk, bind1(1:2,1:2), bind2

	Real(8) eu_k(1:2,1:2), eu_z(1:2,1:2), emax(1:2)
	Real(8), Allocatable :: eer(:,:) 
	End Module Def_Euler

C Definition of module that provides the information for the computation of the DM statistic
	Module Def_DM

	Logical compute_dm, dm_new_shocks
	Integer(4) dm_nobs, dm_nofs, dm_start, dm_lags, avbc
	Real(8) dm_stat(1:4), chi(1:2), kminmax(1:3,1:2), zminmax(1:3,1:2), dm_k(1:2), dm_z(1:2)
	End Module Def_DM


C Module for PE
	Module Def_Proj

	Integer(4) ngamma, proj_type
	Integer(4) psi_initial, psi_increase, psi_type, psi_degree, psi_degree1, psi_degree2
	Integer(4) k_nodes, z_nodes

	Real(8) psi_bounds(1:2,1:2), x_bounds(1:2,1:2)
	Real(8) k_min, k_max, psi_kmin, psi_kmax, psi_zmin, psi_zmax, z_min, z_max
	Real(8) k_ex, z_ex
	Real(8), allocatable :: gamma0(:)
	
	Character(30) f_pe_initial_p

	End Module Def_Proj
	
C Module for errors
	Module Errors

	Integer(2) ErrorCode, FError, LError, lfh


	End Module Errors


C Here begins the program
      program SGNNI_c

	use Def_Par
	use Def_Moments
	use Def_Output
	use Def_Euler
	use Def_DM	
	use Errors

	use DFLIB      
	use IMSLF90 

      Integer(4) i, j	      	
	
	Logical ok, InitialSettings
      
	Type (rccoord) curpos		
      
C Text for about box
      i=AboutBoxQQ('SGNNI_c.exe\r Authors: Burkhard Heer and Alfred Maussner\r Version: Feb., 25th, 2008'C)

C Read method from input file
	Call GetI('algo','Parameters.txt',algo)
	Call GetL('compute_euler','Parameters.txt',compute_euler)
	Call GetI('eu_nz','Parameters.txt',eu_nz)
	Call GetI('eu_nk','Parameters.txt',eu_nk)
	Call GetD('dm_k1','Parameters.txt',dm_k(1))
	Call GetD('dm_k2','Parameters.txt',dm_k(2))
	Call GetD('dm_z1','Parameters.txt',dm_z(1))
	Call GetD('dm_z2','Parameters.txt',dm_z(2))	
	Call GetD('eu_kmin1','Parameters.txt',eu_k(1,1))
	Call GetD('eu_kmin2','Parameters.txt',eu_k(2,1))
	Call GetD('eu_kmax1','Parameters.txt',eu_k(1,2))
	Call GetD('eu_kmax2','Parameters.txt',eu_k(2,2))
	Call GetD('eu_zmin1','Parameters.txt',eu_z(1,1))
	Call GetD('eu_zmin2','Parameters.txt',eu_z(2,1))
	Call GetD('eu_zmax1','Parameters.txt',eu_z(1,2))		
	Call GetD('eu_zmax2','Parameters.txt',eu_z(2,2))
	Call GetL('compute_dm','Parameters.txt',compute_dm)
	Call GetL('dm_new_shocks','Parameters.txt',dm_new_shocks)
	Call GetI('dm_nobs','Parameters.txt',dm_nobs)
	Call GetI('dm_nofs','Parameters.txt',dm_nofs)
	Call GetI('dm_start','Parameters.txt',dm_start)
	Call GetI('dm_lags','Parameters.txt',dm_lags)

      Call GetD('alpha','Parameters.txt',alpha)
      Call GetD('eta','Parameters.txt',eta)
      Call GetD('delta','Parameters.txt',delta)
      Call GetD('beta','Parameters.txt',beta)
      Call GetD('rho','Parameters.txt',rho)
      Call GetD('sigma','Parameters.txt',sigma)      
      	
C Read name of output file from input file
      Call GetS('outfile','Parameters.txt',outfile)

C Prepare output window
      ok=InitialSettings()
      Call MakeSmallWindow(10,"Program Progress",2)      

C Open LogFile
	lfh=12
	open(lfh,file='LogFile.txt')
	WRITE(lfh,"(A50)") MyDate()	
	
Computation of stationary solution
	zstar=0.0d0

	kstar=((1.0d0-beta*(1.0d0-delta))/(alpha*beta))**(1.d0/(alpha-1.d0)) ! stationary level of capital
	ystar=kstar**alpha                  ! stationary level of output
      istar=delta*kstar                   ! stationary level of investment  
      cstar=ystar-istar                   ! stationary level of consumption 	      
	lstar=cstar**(-eta)                 ! stationary marginal utility

C Get Solution
      ErrorCode=0
C Find PE solution
	Call Projection

C Write result to file    
      If (ErrorCode.eq.0) then
		if (compute_dm)    Call DM
		if (compute_euler) Call Euler
	    CALL ElapsedTime(sum(Lsec),LaufZeit)      
		Call WriteResults
	    CALL GETTEXTPOSITION(curpos)
          CALL SETTEXTPOSITION(curpos.row+1,2,curpos)
          WRITE(10,"('Run time= ',A40)") LaufZeit
      else
		Call ClearScreen($GCLEARSCREEN)
		CALL SETTEXTPOSITION(1,1,curpos)
		WRITE(10,"(A)") 'Not able to find a solution or to simulate the model'
		Call SETTEXTPOSITION(2,1,curpos)
		WRITE(10,"(A)") 'See logfile.txt for further information'
	endif


C Close LogFile
	close(lfh)	
      End

C *************************************** Subroutines ***********************************************

	Subroutine WriteResults

		use Def_Par
		use Def_Proj
		use Def_Moments
		use Def_Output
		use Def_Euler
		use Def_DM
		use Errors

		Integer(4) i, j	
		Character(40) temp
C Write solution to file
		open(11,file=trim(OutFile))
		WRITE(11,"(A50)") MyDate()
		WRITE(11,"('Run time:',A40)") Laufzeit
		WRITE(11,"('')")
		Do i=1,3
			Call ElapsedTime(Lsec(i),temp)
			WRITE(11,"(A18,A40)") Laufzeiten(i),temp
		End Do
		WRITE(11,"('')")
		if (compute_euler) then
			Write(11,"('Maximum absolute value of Euler equation residual: ',F20.10)") emax(1)
			Write(11,"('Maximum absolute value of Euler equation residual: ',F20.10)") emax(2)
			Do j=1,2,1
				Write(11,"('eu_zmin(',I1,')=',F10.4)") j, eu_z(j,1)
				Write(11,"('eu_zmax(',I1,')=',F10.4)") j, eu_z(j,2)
			End Do
			Do j=1,2,1
				Write(11,"('eu_kmin(',I1,')=',F10.4)") j, eu_k(j,1)		
				Write(11,"('eu_kmax(',I1,')=',F10.4)") j, eu_k(j,2)
			End Do
			Write(11,"('eu_nz  =',I4)") eu_nz
			Write(11,"('eu_nk  =',I4)") eu_nk
			Write(11,"('')")
			Write(11,"('Binding constraints in computation of Euler residual:')")
			Do j=1,2,1
				Write(11,"(2(I5,2X))") (bind1(j,i), i=1,2)
			End Do			
		endif
		if (compute_dm) then
			Write(11,"('DM-Statistic:')")
			Write(11,"('Number of valid simulations : ',F8.3)") dm_stat(3)
			Write(11,"('Fraction of DM below ',F6.3,' : ',F7.3)") chi(1), (dm_stat(1)/dm_stat(3))
			Write(11,"('Fraction of DM above ',F6.3,' : ',F7.3)") chi(2), (dm_stat(2)/dm_stat(3))
			Write(11,"('dm_nobs                     : ',I7)")  dm_nobs
			Write(11,"('dm_nofs                     : ',I7)")  dm_nofs
			Write(11,"('dm_lags                     : ',I7)")  dm_lags
			Write(11,"('dm_start                    : ',I7)")  dm_start
			Do j=1,3,1
				Write(11,"('dm_zmin(',I1,')=',F10.4)") j, zminmax(j,1)
				Write(11,"('dm_zmax(',I1,')=',F10.4)") j, zminmax(j,2)
			End Do
			Do j=1,3,1
				Write(11,"('dm_kmin(',I1,')=',F10.4)") j, kminmax(j,1)		
				Write(11,"('dm_kmax(',I1,')=',F10.4)") j, kminmax(j,2)
			End Do		
			Write(11,"('avg. numb. of bdg. constr.  : ',F7.2)") dble(avbc/dm_nofs)
			Write(11,"('')")
		endif
		WRITE(11,"(A)") ' '	
		WRITE(11,"(A)") 'Parameters of the model:'		
		WRITE(11,"('alpha=   ',F10.4)")  alpha
		WRITE(11,"('beta=    ',F10.4)")  beta 
		WRITE(11,"('eta=     ',F10.4)")  eta				
		WRITE(11,"('delta=   ',F10.4)")  delta
		WRITE(11,"('rho=     ',F10.4)")  rho
		WRITE(11,"('sigma=   ',F10.5)")  sigma
		WRITE(11,"(A)") ' '
		Call WriteCoefPE()
		Write(11,"(A)") ' '
		Write(11,"('Parameters of the Algorithm:')")
		Call RetrievSettings
	
		close(11)
    
	open(15,file='Summary.txt',access='append')
	Select Case(Psi_type)

		Case(1)
			write(15,"('$p$= ',I1,' & ',E10.4,' & ',E10.4,' & ',F4.1,' & ',F4.1)") psi_degree,emax(1), emax(2),
     +        (100*(dm_stat(1)/dm_stat(3))), (100*(dm_stat(2)/dm_stat(3)))

		Case(2)
			write(15,"('$p_1$= ',I1,' $p_2$= ',I1,' & ',E10.4,' & ',E10.4,' & ',F4.1,' & ',F4.1)") psi_degree1, psi_degree2
     +			,emax(1), emax(2), (100*(dm_stat(1)/dm_stat(3))), (100*(dm_stat(2)/dm_stat(3)))
	End Select
	close(15)
	Return

	End Subroutine

C WriteCoefPE
	Subroutine WriteCoefPE()

	use Def_Proj

	Integer(4) i, j, l

	Write(11,"('Coefficients of Psi:')")

	write(11,"('gamma(i,j): Index i for K and index j for Z')")	
	
	Select Case(psi_type)
	
		Case(1)		
			l=1
			Do i=1,psi_degree,1
				Do j=1,(psi_degree+1-i),1					
					write(11,"('gamma(',I1,',',I2,')= ',F15.10)") (i-1),(j-1), gamma0(l)
					l=l+1
				End Do
			End Do

		Case(2)
			l=1
			Do i=1,psi_degree1,1
				Do j=1,psi_degree2,1
					write(11,"('gamma(',I1,',',I2,')= ',F15.10)") (i-1),(j-1), gamma0(l)
					l=l+1
				End Do
			End Do

	End Select
	

	Return

	End Subroutine

C  Retrieve Settings

      Subroutine RetrievSettings()

	use Def_Par
	use Def_Proj

      Integer(4) fhi, fho
      Character(256) Text1
	
	fhi=14
	fho=11

	open(14,file='Parameters.txt')

	Write(11,"(A)") '[Proj]'      
	Do while (.not. eof(14))
	    Read(14,'(A)') Text1	
          If (Text1(1:6).eq.'[Proj]') Call WriteFileSequence(fhi,fho)
	End Do
	Rewind(14)
	write(11,'(A)') ' '
	write(11,"(A)") '[NLEQ]'
	Do while (.not. eof(14))
	    Read(14,'(A)') Text1	
		if (Text1(1:6).eq.'[NLEQ]') Call WriteFileSequence(fhi,fho)			    
	EndDo
	Rewind(14)
	write(11,'(A)') ' '
	write(11,"(A)") '[Search]'
	Do while (.not. eof(14))
	    Read(14,'(A)') Text1	
		if (Text1(1:8).eq.'[Search]') Call WriteFileSequence(fhi,fho)			    
	EndDo
	if (psi_initial.eq.3) then
		Rewind(14)
		write(11,'(A)') ' '
		write(11,"(A)") '[Linear]'
		Do while (.not.eof(14))
			Read(14,'(A)') Text1
			if (Text1(1:8).eq.'[Linear]') Call WriteFileSequence(fhi,fho)
		EndDo
	Endif
	close(14)		
	Return

	End Subroutine


	Subroutine WriteFileSequence(fhi,fho)

	Integer(4) fhi,fho

	Character(256) Text2

	Read(fhi,'(A)') Text2
	Do while(Text2(1:1).ne.' ')
		Write(fho,'(A)') Text2
		Read(fhi,'(A)') Text2
	End Do

	Return

	End Subroutine

